<?php
class MainWP_Post_Plus_Clone {
	private static $instance = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_Post_Plus_Clone
	 */
	static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * Runs each time the class is called.
	 */
	public function __construct() {
	}

	/**
	 * Admin Init
	 *
	 * Initiates admin hooks.
	 */
	public function admin_init() {
		do_action( 'mainwp_ajax_add_action', 'mainwp_post_plus_get_clone', array( $this, 'ajax_post_get_clone' ) );
		do_action( 'mainwp_ajax_add_action', 'mainwp_post_plus_get_bulk_clone', array( $this, 'ajax_post_get_bulk_clone' ) );
		do_action( 'mainwp_ajax_add_action', 'mainwp_post_plus_preparing_bulk_clone', array( $this, 'ajax_preparing_bulk_clone' ) );
		do_action( 'mainwp_ajax_add_action', 'mainwp_post_plus_posting_bulk_clone', array( $this, 'ajax_posting_bulk_clone' ) );
		do_action( 'mainwp_ajax_add_action', 'mainwp_post_plus_bulk_clone_delete', array( $this, 'ajax_bulk_clone_delete' ) );
		add_filter( 'mainwp_manageposts_get_post_result', array( $this, 'hook_get_post_result' ), 10, 3 );
		add_action( 'mainwp_manage_pages_sidebar_bottom', array( &$this, 'hook_manage_pages_sidebar_bottom' ) );
		add_action( 'mainwp_manage_posts_sidebar_bottom', array( &$this, 'hook_manage_posts_sidebar_bottom' ) );
		add_action( 'mainwp_manage_pages_action_item', array( &$this, 'hook_manage_pages_action_item' ) );
		add_action( 'mainwp_manage_posts_action_item', array( &$this, 'hook_manage_posts_action_item' ) );
		add_action( 'mainwp_manage_pages_bulk_action', array( &$this, 'hook_manage_pages_bulk_action' ) );
		add_action( 'mainwp_manage_posts_bulk_action', array( &$this, 'hook_manage_posts_bulk_action' ) );
		$this->check_to_clone();
	}

	/**
	 * Method check_to_clone()
	 *
	 * Check to clone post.
	 * check_to_clone
	 *
	 * @return bool true or false
	 */
	public function check_to_clone() {
		if ( isset( $_GET['postplus_nonce'] ) && wp_verify_nonce( sanitize_key( $_GET['postplus_nonce'] ), 'post_plus_clone' ) ) {
			$dash_post_id    = intval( $_GET['id'] );
			$selected_sites  = array();
			$selected_groups = array();

			if ( isset( $_GET['selected_sites'] ) && ! empty( $_GET['selected_sites'] ) ) {
				$selected_sites = explode( '-', sanitize_text_field( wp_unslash( $_GET['selected_sites'] ) ) ); // sanitize ok.
				$selected_sites = array_map( 'intval', $selected_sites );
				$selected_sites = array_filter( $selected_sites );
			}

			if ( isset( $_GET['selected_groups'] ) && ! empty( $_GET['selected_groups'] ) ) {
				$selected_groups = explode( '-', sanitize_text_field( wp_unslash( $_GET['selected_groups'] ) ) ); // sanitize ok.
				$selected_groups = array_map( 'intval', $selected_groups );
				$selected_groups = array_filter( $selected_groups );
			}

			if ( empty( $selected_sites ) && empty( $selected_groups ) ) {
				return false;
			}

			$select_by = 'site';
			if ( ! empty( $selected_sites ) ) {
			} elseif ( ! empty( $selected_groups ) ) {
				$select_by = 'group';
			}

			update_post_meta( $dash_post_id, '_selected_sites', $selected_sites );
			update_post_meta( $dash_post_id, '_selected_groups', $selected_groups );
			update_post_meta( $dash_post_id, '_selected_by', $select_by );
			delete_post_meta( $dash_post_id, '_mainwp_edit_post_id' ); // to clone, not update.

			$saved_post_type = get_post_meta( $dash_post_id, '_mainwp_edit_post_type', true );
			if ( ! empty( $saved_post_type ) ) {
				update_post_meta( $dash_post_id, '_mainwp_edit_post_save_to_post_type', $saved_post_type );
			}

			$saved_post_status = get_post_meta( $dash_post_id, '_mainwp_edit_post_status', true );
			if ( ! empty( $saved_post_status ) ) {
				update_post_meta( $dash_post_id, '_edit_post_status', $saved_post_status ); // to save post_status.
			}

			$post_type = get_post_type( $dash_post_id );

			if ( 'bulkpost' == $post_type ) {
				wp_safe_redirect( admin_url( 'admin.php?page=PostingBulkPost&posting_nonce=' . wp_create_nonce( 'posting_nonce_' . $dash_post_id ) . '&id=' . $dash_post_id ) );
			} elseif ( 'bulkpage' == $post_type ) {
				wp_safe_redirect( admin_url( 'admin.php?page=PostingBulkPage&posting_nonce=' . wp_create_nonce( 'posting_nonce_' . $dash_post_id ) . '&id=' . $dash_post_id ) );
			}
			die();
		}
		return true;
	}

	/**
	 * Method check_security()
	 *
	 * @return bool true or false
	 */
	public function check_security() {
		$action = isset( $_POST['action'] ) ? $_POST['action'] : '';
		if ( empty( $action ) ) {
			die( -1 );
		}
		do_action( 'mainwp_secure_request', $action );
	}

	/**
	 * Method ajax_post_get_clone()
	 *
	 * Get post to clone.
	 */
	public function ajax_post_get_clone() {
		$this->check_security();
		do_action( 'mainwp_manage_posts_get_edit' );
		die();
	}

	/**
	 * Method ajax_post_get_bulk_clone()
	 *
	 * Get post to clone.
	 */
	public function ajax_post_get_bulk_clone() {
		$this->check_security();
		do_action( 'mainwp_manage_posts_get_edit' );
		die();
	}

	/**
	 * Method hook_get_post_result()
	 *
	 * Get post result.
	 *
	 * @param array $result result created post.
	 * @param array $post_data post get info.
	 */
	public function hook_get_post_result( $result, $post_data = array() ) {
		if ( is_array( $result ) && isset( $result['id'] ) ) {
			$dash_post_id      = $result['id'];
			$post              = get_post( $dash_post_id );
			$result['title']   = $post->post_title;
			$result['content'] = $post->post_content;

			$detected_post_builder = false;
			if ( $post && $this->detect_post_builder_custom_content( $post->post_content, true ) ) {
				$detected_post_builder = true;
			}

			if ( ! $detected_post_builder ) {
				$post_custom          = get_post_custom( $dash_post_id );
				$post_custom_filtered = array();
				foreach ( $post_custom as $mkey => $mval ) {
					if ( false === stripos( $mkey, 'mainwp_' ) ) {
						$post_custom_filtered[ $mkey ] = $mval; // skip custom data with "mainwp_" in key.
					}
				}

				$post_custom_string = print_r( $post_custom_filtered, true ); // array to string.
				if ( $this->detect_post_builder_custom_content( $post_custom_string ) ) { // check for custom post fields.
					$detected_post_builder = true;
				}
			}

			if ( $detected_post_builder ) {
				$result['postplus_warning'] = esc_html__( 'This may include custom post builder content, and cloning may incorrectly.', 'mainwp' );
			}

			$dts            = strtotime( $post->post_modified_gmt );
			$result['date'] = MainWP_Post_Plus::format_timestamp( MainWP_Post_Plus::get_timestamp( $dts ) );
			$new_post       = json_decode( base64_decode( $post_data['new_post'] ), true ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions -- base64_encode used for http encoding compatible.
			if ( isset( $new_post['post_type'] ) && ! empty( $new_post['post_type'] ) ) {
				$result['posttype'] = $new_post['post_type'];
				update_post_meta( $dash_post_id, '_mainwp_edit_post_type', $new_post['post_type'] );
			} else {
				$result['posttype'] = $post->post_type;
			}
			update_post_meta( $dash_post_id, '_mainwp_edit_post_status', $post->post_status );
			$result['status'] = self::get_status( $post->post_status );
		}
		return $result;
	}

	/**
	 * Method detect_post_builder_custom_content()
	 *
	 * Get Post status.
	 *
	 * @param mixed $content Post content.
	 * @param bool  $check_http_str Check 'http' string.
	 *
	 * @return string $status Post status.
	 */
	private static function detect_post_builder_custom_content( $content, $check_http_str = false ) {
		$matches      = array();
		$foundMatches = preg_match_all( '#"[^"]*.(png|gif|jpg|jpeg)"#ix', $content, $matches, PREG_SET_ORDER );
		if ( 0 < $foundMatches ) {
			if ( $check_http_str ) {
				foreach ( $matches as $matche ) {
					$matche_0 = $matche[0];
					if ( false === stripos( $matche_0, 'http' ) ) {
						return true; // not found 'http', it may be builder custom post.
					}
				}
			} else {
				return true;
			}
		}

		$matches      = array();
		$foundMatches = preg_match_all( '#<iframe#ix', $content, $matches, PREG_SET_ORDER );
		if ( 0 < $foundMatches ) {
			return true;
		}
		return false;
	}

	/**
	 * Method get_status()
	 *
	 * Get Post status.
	 *
	 * @param mixed $status Post status.
	 *
	 * @return string $status Post status.
	 */
	private static function get_status( $status ) {
		if ( 'publish' === $status ) {
			return 'Published';
		}

		return esc_html( ucfirst( $status ) );
	}

	public function hook_manage_pages_action_item() {
		?>
		<a class="item page_getclone" href="#"><?php esc_html_e( 'Clone', 'mainwp-post-plus-extension' ); ?></a>
		<?php
	}

	public function hook_manage_posts_action_item() {
		?>
		<a class="item post_getclone" href="#"><?php esc_html_e( 'Clone', 'mainwp-post-plus-extension' ); ?></a>
		<?php
	}

	public function hook_manage_pages_bulk_action() {
		?>
		<option value="get_clone"><?php esc_html_e( 'Clone', 'mainwp-post-plus-extension' ); ?></option>
		<?php
	}

	public function hook_manage_posts_bulk_action() {
		?>
		<option value="get_clone"><?php esc_html_e( 'Clone', 'mainwp-post-plus-extension' ); ?></option>
		<?php
	}

	public function hook_manage_pages_sidebar_bottom() {
		self::render_clone_popup( 'bulkpage' );
	}

	public function hook_manage_posts_sidebar_bottom() {
		self::render_clone_popup( 'bulkpost' );
	}

	/**
	 * Method render_clone_popup()
	 *
	 * Popup for clone post.
	 */
	public static function render_clone_popup( $post_type = 'bulkpost' ) {

		$sites_settings = array(
			'type'            => 'checkbox',
			'show_group'      => true,
			'show_select_all' => true,
			'class'           => '',
			'style'           => '',

		);
		$selected_sites  = array();
		$selected_groups = array();
		?>
		<div class="ui large modal" id="postplus-manageposts-modal" action-bulk="no" post-type="<?php echo esc_html( $post_type ); ?>">
		<i class="close icon"></i>
			<div class="header">
				<?php echo ( $post_type == 'bulkpost' ) ? esc_html__( 'Clone Post', 'mainwp' ) : esc_html__( 'Clone Page', 'mainwp' ); ?>
			</div>
			<div class="ui green progress mainwp-pplus-modal-progress" style="display:none">
				<div class="bar"><div class="progress"></div></div>
				<div class="label"></div>
			</div>
			<div class="scrolling content">
				<div class="ui grid">
					<div class="twelve wide column modal-left-content" style="border-right: 1px solid rgba(34,36,38,.15)">
					<div class="ui message" id="mainwp-message-zone" style="display:none"></div>					
						<div id="content_clone_posts_short_detail" style="display:none">							
							<h3 style="margin-top:0" id="post_clone_title"></h3>
								<table class="ui definition table">
									<tbody>
										<tr>
									<td><?php esc_html_e( 'Status', 'mainwp' ); ?></td>
									<td id="post_clone_status"></td>
									</tr>
										<tr>
									<td><?php esc_html_e( 'Date', 'mainwp' ); ?></td>
									<td id="post_clone_date"></td>
									</tr>
										<tr>
									<td><?php esc_html_e( 'Type', 'mainwp' ); ?></td>
									<td id="post_clone_posttype"></td>
									</tr>
									</tbody>
								</table>
								<div class="ui secondary segment" id="post_clone_content"></div>
						</div>
						<div id="content_clone_posts_short_list"  style="display:none">
						</div>
					</div>
					<div class="four wide column popup-side-right">
						<h3><?php esc_html_e( 'Select Sites', 'mainwp' ); ?></h3>
						<?php do_action( 'mainwp_select_sites_box', '', 'checkbox', true, true, '', '', $selected_sites, $selected_groups ); ?>
						<input type="hidden" name="select_sites_nonce" id="select_sites_nonce" value="" />
					</div>
					</div>
						<input type="hidden" id="post_clone_id" value=""/>
					</div>
			<div class="actions">
				<div class="ui two columns grid">
					<div class="left aligned column"><div class="mainwp-modal-close ui cancel button"><?php esc_html_e( 'Cancel', 'mainwp' ); ?></div></div>
					<div class="right aligned column">
						<input type="button" name="publish" id="postplus-publishing-button" class="ui green button" value="<?php esc_attr_e( 'Clone to Selected Sites', 'mainwp' ); ?>">
					</div>
				</div>			
			</div>
		</div>
		<input type="hidden" id="post_plus_clone_nonce" value="<?php echo wp_create_nonce( 'post_plus_clone' ); ?>" />
		
		<div id="content_clone_posts_short_temp" style="display:none">
			<h3 style="margin-top:0" class="post_bulk_clone_title"></h3>
			<div style="display:none" class="post_bulk_clone_message"></div>		
			<div class="post-clone-item">					
					<table class="ui definition table">
						<tbody>
							<tr>
								<td><?php esc_html_e( 'Status', 'mainwp' ); ?></td>
								<td class="post_bulk_clone_status"></td>
								</tr>
									<tr>
								<td><?php esc_html_e( 'Date', 'mainwp' ); ?></td>
								<td class="post_bulk_clone_date"></td>
								</tr>
									<tr>
								<td><?php esc_html_e( 'Type', 'mainwp' ); ?></td>
								<td class="post_bulk_clone_posttype"></td>
							</tr>
						</tbody>
					</table>
			</div>			
		</div>
		<?php
	}

	public function ajax_preparing_bulk_clone() {
		$this->check_security();

		$selected_sites  = isset( $_POST['sites'] ) ? $_POST['sites'] : array();
		$selected_groups = isset( $_POST['groups'] ) ? $_POST['groups'] : array();

		global $mainWPPostPlusExtensionActivator;
		$dbwebsites = apply_filters( 'mainwp_getdbsites', $mainWPPostPlusExtensionActivator->get_child_file(), $mainWPPostPlusExtensionActivator->get_child_key(), $selected_sites, $selected_groups );

		if ( ! empty( $dbwebsites ) ) {
			?>
			<div class="ui relaxed list" id="sites-bulk-clone">
			<?php
			foreach ( $dbwebsites as $website ) {
				?>
				<div class="item site-item-processing queue" website-id="<?php echo intval( $website->id ); ?>" website-name="<?php echo esc_html( $website->name ); ?>"><a href="<?php echo admin_url( 'admin.php?page=managesites&dashboard=' . $website->id ); ?>"><?php echo stripslashes( $website->name ); ?></a>
					<div class="right floated content site-progress"><i class="clock outline icon"></i></div>
				</div>
			<?php } ?>
			</div>
			<?php
		}
		die();
	}

	/**
	 * Method ajax_posting_bulk_clone()
	 *
	 * Bulk clone.
	 */
	public function ajax_posting_bulk_clone() {
		$this->check_security();
		$dash_post_id = isset( $_POST['post_id'] ) && $_POST['post_id'] ? intval( $_POST['post_id'] ) : false;
		delete_post_meta( $dash_post_id, '_mainwp_edit_post_id' ); // to clone, not update.
		$saved_post_type = get_post_meta( $dash_post_id, '_mainwp_edit_post_type', true );
		if ( ! empty( $saved_post_type ) ) {
			update_post_meta( $dash_post_id, '_mainwp_edit_post_save_to_post_type', $saved_post_type );
		}
		update_post_meta( $dash_post_id, '_mainwp_replace_advance_img', true );
		$saved_post_status = get_post_meta( $dash_post_id, '_mainwp_edit_post_status', true );
		if ( ! empty( $saved_post_status ) ) {
			update_post_meta( $dash_post_id, '_edit_post_status', $saved_post_status ); // to save post_status.
		}
		do_action( 'mainwp_manage_posts_bulk_posting' );
		die();
	}

	/**
	 * Method ajax_bulk_clone_delete()
	 *
	 * Delete bulkpost cloned.
	 */
	public function ajax_bulk_clone_delete() {
		$this->check_security();
		$dash_post_id = isset( $_POST['post_id'] ) && $_POST['post_id'] ? intval( $_POST['post_id'] ) : false;
		wp_delete_post( $dash_post_id, true );
		die( 'ok' );
	}
}
