<?php
class MainWP_Post_Plus {
	private static $instance = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_Post_Plus
	 */
	static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * Runs each time the class is called.
	 */
	public function __construct() {
	}

	/**
	 * Admin Init
	 *
	 * Initiates admin hooks.
	 */
	public function admin_init() {
		do_action( 'mainwp_ajax_add_action', 'mainwp_pplus_delete_post', array( $this, 'ajax_delete_post' ) );
		do_action( 'mainwp_ajax_add_action', 'mainwp_pplus_publish_post', array( $this, 'ajax_posting' ) );
		add_action( 'mainwp_bulkpost_edit', array( &$this, 'postplus_metabox' ), 10, 2 );
		add_filter( 'post_updated_messages', array( &$this, 'post_updated_messages' ) );
		add_action( 'mainwp_bulkpost_categories_handle', array( $this, 'postplus_categories_handle' ), 10, 2 ); // deprecated?
		add_action( 'mainwp_bulkpost_tags_handle', array( $this, 'postplus_tags_handle' ), 10, 3 );
		add_filter( 'mainwp_bulkpost_saved_categories', array( &$this, 'postplus_saved_categories' ), 10, 2 );
		add_action( 'mainwp_save_bulkpost', array( &$this, 'postplus_metabox_handle' ) );
		add_action( 'mainwp_save_bulkpage', array( &$this, 'postplus_metabox_handle' ) );
		add_filter( 'mainwp_posting_bulkpost_post_status', array( &$this, 'posting_post_status' ), 10, 2 );
		add_filter( 'mainwp_after_posting_bulkpost_result', array( &$this, 'after_posting_bulk' ), 10, 4 );
		add_filter( 'mainwp_after_posting_bulkpage_result', array( &$this, 'after_posting_bulk' ), 10, 4 );
		add_action( 'mainwp_edit_posts_after_submit_button', array( &$this, 'posts_after_submit_button' ) );
		add_filter( 'mainwp_header_actions_right', array( $this, 'screen_options' ), 10, 2 );

		$this->handle_sites_screen_settings();
	}

	/**
	 * Method get_status()
	 *
	 * Get Post status.
	 *
	 * @param mixed $status Post status.
	 *
	 * @return string $status Post status.
	 */
	private static function get_status( $status ) {
		if ( 'publish' === $status ) {
			return __( 'Published', 'mainwp-post-plus-extension' );
		}

		return esc_html( ucfirst( $status ) );
	}

	/**
	 * Get Sub-pages Post
	 *
	 * Gets sub-page post data.
	 *
	 * @param array $args Post data array.
	 *
	 * @return array $args Updated post data array.
	 */
	public function get_sub_pages_post( $args ) {
		$args[] = array(
			'slug'     => 'Draft',
			'title'    => 'Drafts',
			'callback' => array( &$this, 'render_draft_post' ),
		);
		return $args;
	}

	/**
	 * Get Sub-pages Page
	 *
	 * Gets sub-page page data.
	 *
	 * @param array $args Page data array.
	 *
	 * @return array $args Updated page data array.
	 */
	public function get_sub_pages_page( $args ) {
		$args[] = array(
			'slug'     => 'Draft',
			'title'    => 'Drafts',
			'callback' => array( &$this, 'render_draft_page' ),
		);
		return $args;
	}

	/**
	 * Delete Post
	 *
	 * Deletes post (MainWP Draft).
	 */
	public function ajax_delete_post() {
		do_action( 'mainwp_secure_request', 'mainwp_pplus_delete_post' );
		$post_id = isset( $_POST['postId'] ) ? $_POST['postId'] : 0;
		if ( ! empty( $post_id ) && wp_delete_post( $post_id ) ) {
			die( 'success' );
		}
		die( 'failed' );
	}

	/**
	 * Post Updated Message
	 *
	 * Sets the new feedback message that shows after successful draft update.
	 *
	 * @param array $messages Messages array.
	 *
	 * @return array $messages Updated messages array.
	 */
	public function post_updated_messages( $messages ) {
		$messages['post'][96] = __( 'Draft updated successfully!', 'mainwp-post-plus-extension' );
		return $messages;
	}

	public function postplus_categories_handle( $post_id, $categories ) {
		if ( $post_id ) {
			update_post_meta( $post_id, '_saved_draft_categories', base64_encode( implode( ', ', $categories ) ) );
		}
	}

	public function postplus_tags_handle( $post_id, $post_type, $tags ) {
		$post = get_post( $post_id );
		if ( ! empty( $post ) && $post->post_type == $post_type ) {
			update_post_meta( $post_id, '_saved_draft_tags', base64_encode( $tags ) );
		}
	}

	public function postplus_metabox( $post, $post_type ) {
		require_once MAINWP_POST_PLUS_PLUGIN_DIR . 'includes/post-plus-metabox.php';
	}

	public function check_boilerplate_saving( $post_id ) {
		if ( ! isset( $_POST['mainwp_boilerplate_nonce'] ) || ! wp_verify_nonce( $_POST['mainwp_boilerplate_nonce'], 'boilerplate_' . $post_id ) ) {
			return false;
		}
		return true;
	}

	function posts_after_submit_button() {
		// this button will support saving posts/pages to sites as draft posts/pages
		?>
			<div class="mainwp-search-submit">
				<input type="submit" name="postplus_save_as_draft" id="publish" class="ui big fluid button" value="<?php esc_attr_e( 'Save as Draft', 'mainwp' ); ?>">
			</div>
		<?php
	}

	public function postplus_metabox_handle( $post_id ) {

		if ( ! isset( $_POST['mainwp_pplus_metabox_submit'] ) ) {
			return;
		}

		$post = get_post( $post_id );

		update_post_meta( $post_id, '_mainwp_post_plus', 'yes' ); // this meta will use to process on child

		$privelege = isset( $_POST['pplus_meta_privelege'] ) && is_array( $_POST['pplus_meta_privelege'] ) ? $_POST['pplus_meta_privelege'] : '';

		update_post_meta( $post_id, '_saved_draft_random_privelege', ( is_array( $privelege ) ? base64_encode( wp_json_encode( $privelege ) ) : '' ) );

		if ( $post->post_type != 'bulkpage' ) {
			$cat = isset( $_POST['pplus_meta_random_category'] ) ? 1 : '';
			update_post_meta( $post_id, '_saved_draft_random_category', $cat );
		}

		$publish_date = isset( $_POST['pplus_meta_random_publish_date'] ) ? 1 : '';
		update_post_meta( $post_id, '_saved_draft_random_publish_date', $publish_date );

		$date_from = $date_to = '';

		if ( ! empty( $publish_date ) ) {
			$date_from = isset( $_POST['pplus_random_date_from'] ) ? trim( $_POST['pplus_random_date_from'] ) : 0;
			if ( ! empty( $date_from ) ) {
				$date_from = strtotime( $date_from );
			}
			$date_to = isset( $_POST['pplus_random_date_to'] ) ? trim( $_POST['pplus_random_date_to'] ) : 0;
			if ( ! empty( $date_to ) ) {
				$date_to  = strtotime( $date_to );
				$date_to += 24 * 3600 - 1;
			}
		}

		update_post_meta( $post_id, '_saved_draft_publish_date_from', $date_from );
		update_post_meta( $post_id, '_saved_draft_publish_date_to', $date_to );


		if ( $post->post_status == 'draft' ) {
			if ( ! $this->check_boilerplate_saving( $post_id ) ) {
				update_post_meta( $post_id, '_saved_as_draft', 'yes' );
			}
		}

		$draft = '';
		if ( isset( $_POST['postplus_save_as_draft'] ) ) {
			$draft = 'draft';
		}

		update_post_meta( $post_id, '_postplus_edit_post_status', $draft );
	}

	public function posting_post_status( $post_status, $post_id ) {

		$draft_status = get_post_meta( $post_id, '_postplus_edit_post_status', true );
		if ( $draft_status == 'draft' ) {
			$post_status = 'draft';
		}

		return $post_status;
	}

	public function after_posting_bulk( $input, $post, $dbwebsites, $output ) {

		if ( $post ) {

			$saved_draft = get_post_meta( $post->ID, '_saved_as_draft', true );
			if ( $saved_draft == 'yes' ) {

				$failed_posts = array();
				foreach ( $dbwebsites as $website ) {
					if ( isset( $output->ok[ $website->id ] ) && ( $output->ok[ $website->id ] == 1 ) && ( isset( $output->added_id[ $website->id ] ) ) ) {
						// ok, success
					} else {
						$failed_posts[] = $website->id;
					}
				}
				$del_post = true;
				if ( count( $failed_posts ) > 0 ) {
					$del_post = false;
					update_post_meta( $post->ID, '_selected_sites', base64_encode( serialize( $failed_posts ) ) );
					update_post_meta( $post->ID, '_selected_groups', '' );
					wp_update_post(
						array(
							'ID'          => $post->ID,
							'post_status' => 'draft',
						)
					);
				}

				if ( ! $del_post ) {
					update_post_meta( $post->ID, '_bulkpost_do_not_del', 'yes' ); // this will prevent delete the post after about posing
				}
			}
		}

		return $input; // return false here
	}

	public function postplus_saved_categories( $post, $categories = array() ) {
		if ( ! empty( $post ) ) {
			$cats = get_post_meta( $post->ID, '_saved_draft_categories', true );
			$cats = base64_decode( $cats );
			$cats = explode( ', ', $cats );
			if ( ! empty( $cats ) ) {
				$categories = array_merge( $categories, $cats );
			}
		}
		return $categories;
	}

	public function ajax_posting() {
		do_action( 'mainwp_secure_request', 'mainwp_pplus_publish_post' );
		$post_id = isset( $_POST['postId'] ) ? $_POST['postId'] : 0;
		$error   = '';
		if ( $post_id ) {
			try {
				$result = $this->draft_posting( $post_id );
				if ( ! empty( $result ) ) {
					die( json_encode( $result ) );
				}
			} catch ( Exception $ex ) {
				$error = $ex->getMessage();
			}
		}

		if ( ! empty( $error ) ) {
			die( json_encode( array( 'error' => $error ) ) );
		}

		die();
	}

	public static function posting_bulk_handler( $data, $website, &$output ) {
		if ( preg_match( '/<mainwp>(.*)<\/mainwp>/', $data, $results ) > 0 ) {
			$result      = $results[1];
			$information = json_decode( base64_decode( $result ), true );

			if ( isset( $information['added'] ) ) {
				$output->ok[ $website->id ] = '1';
				if ( isset( $information['link'] ) ) {
					$output->link[ $website->id ] = $information['link'];
				}
				if ( isset( $information['added_id'] ) ) {
					$output->added_id[ $website->id ] = $information['added_id'];
				}
			} elseif ( isset( $information['error'] ) ) {
				$output->errors[ $website->id ] = $information['error'];
			} else {
				$output->errors[ $website->id ] = __( 'Undefined error occurred. Please, try again. If the issue continues, please contact the MainWP support.', 'mainwp-post-plus-extension' );
			}
		} else {
			$output->errors[ $website->id ] = apply_filters( 'mainwp_getErrorMessage', 'NOMAINWP', $website->url );

		}
	}

	public function draft_posting( $id ) {
		$post = get_post( $id );
		if ( $post ) {
			$selected_by     = get_post_meta( $id, '_selected_by', true );
			$selected_sites  = unserialize( base64_decode( get_post_meta( $id, '_selected_sites', true ) ) );
			$selected_groups = unserialize( base64_decode( get_post_meta( $id, '_selected_groups', true ) ) );

			if ( empty( $selected_by ) || ( empty( $selected_sites ) && empty( $selected_groups ) ) ) {
				throw new Exception( __( 'No websites selected. Please select at least one child site or group.' ) );
				return;
			}

			$post_category = base64_decode( get_post_meta( $id, '_categories', true ) );

			$post_tags   = base64_decode( get_post_meta( $id, '_tags', true ) );
			$post_slug   = base64_decode( get_post_meta( $id, '_slug', true ) );
			$post_custom = get_post_custom( $id );

			$galleries           = get_post_gallery( $id, false );
			$post_gallery_images = array();

			if ( is_array( $galleries ) && isset( $galleries['ids'] ) ) {
				$attached_images = explode( ',', $galleries['ids'] );
				foreach ( $attached_images as $attachment_id ) {
					$attachment = get_post( $attachment_id );
					if ( $attachment ) {
							$post_gallery_images[] = array(
								'id'          => $attachment_id,
								'alt'         => get_post_meta( $attachment->ID, '_wp_attachment_image_alt', true ),
								'caption'     => $attachment->post_excerpt,
								'description' => $attachment->post_content,
								'src'         => $attachment->guid,
								'title'       => $attachment->post_title,
							);
					}
				}
			}

			include_once ABSPATH . 'wp-includes' . DIRECTORY_SEPARATOR . 'post-thumbnail-template.php';

			$post_featured_image = get_post_thumbnail_id( $id );
			$mainwp_upload_dir   = wp_upload_dir();

			$new_post = array(
				'post_title'     => $post->post_title,
				'post_content'   => $post->post_content,
				'post_status'    => 'publish',
				'post_date'      => $post->post_date,
				'post_date_gmt'  => $post->post_date_gmt,
				'post_tags'      => $post_tags,
				'post_name'      => $post_slug,
				'post_excerpt'   => $post->post_excerpt,
				'comment_status' => $post->comment_status,
				'ping_status'    => $post->ping_status,
				'id_spin'        => $post->ID,
			);

			if ( $post->post_type == 'bulkpage' ) {
				$new_post['post_type'] = 'page';
				unset( $new_post['post_tags'] );
			}

			if ( null != $post_featured_image ) { // Featured image is set, retrieve URL
				$img                 = wp_get_attachment_image_src( $post_featured_image, 'full' );
				$post_featured_image = $img[0];
			}

			global $mainWPPostPlusExtensionActivator;
			$dbwebsites = apply_filters( 'mainwp_getdbsites', $mainWPPostPlusExtensionActivator->get_child_file(), $mainWPPostPlusExtensionActivator->get_child_key(), $selected_sites, $selected_groups );

			$output           = new stdClass();
			$output->ok       = array();
			$output->errors   = array();
			$output->link     = array();
			$output->added_id = array();

			if ( count( $dbwebsites ) > 0 ) {
				$post_data = array(
					'new_post'            => base64_encode( wp_json_encode( $new_post ) ),
					'post_custom'         => base64_encode( wp_json_encode( $post_custom ) ),
					'post_category'       => base64_encode( $post_category ),
					'post_featured_image' => base64_encode( $post_featured_image ),
					'post_gallery_images' => base64_encode( wp_json_encode( $post_gallery_images ) ),
					'mainwp_upload_dir'   => base64_encode( wp_json_encode( $mainwp_upload_dir ) ),
				);

				if ( $post->post_type == 'bulkpage' ) {
					unset( $post_data['post_category'] );
				}

				do_action( 'mainwp_fetchurlsauthed', $mainWPPostPlusExtensionActivator->get_child_file(), $mainWPPostPlusExtensionActivator->get_child_key(), $dbwebsites, 'newpost', $post_data, array( 'MainWP_Post_Plus', 'posting_bulk_handler' ), $output );
			}

			$failed_posts = array();

			foreach ( $dbwebsites as $website ) {
				if ( isset( $output->added_id[ $website->id ] ) && $output->ok[ $website->id ] == 1 ) {
					do_action( 'mainwp_bulkposting_done', $post, $website, $output );
				} else {
					$failed_posts[] = $website->id;
				}
			}

			if ( count( $failed_posts ) == 0 ) {
				wp_delete_post( $id, true );
			} else {
				update_post_meta( $post->ID, '_selected_sites', base64_encode( serialize( $failed_posts ) ) );
				update_post_meta( $post->ID, '_selected_groups', '' );
			}

			$result  = '';
			$result .= '<div class="ui list">';
			foreach ( $dbwebsites as $website ) {
				$result .= '<div class="item">';
				$result .= ( isset( $output->ok[ $website->id ] ) && $output->ok[ $website->id ] == 1 ? '<i class="green check icon"></i> ' . $website->name . ' - ' . __( 'New post created successfully!', 'mainwp-post-plus-extension' ) : '<i class="red times icon"></i> ' . $website->name . ' - ' . $output->errors[ $website->id ] );
				$result .= '</div>';
			}
			$result .= '</div>';

			return array(
				'result'       => $result,
				'failed_posts' => count( $failed_posts ),
			);
		}
		return false;
	}

	/**
	 * Render Posts Drafts pages
	 *
	 * Renders the posts drafts page.
	 */
	public function render_draft_post() {
		do_action( 'mainwp_pageheader_post', 'Draft' );
		echo self::render_drafts_list( 'bulkpost' );
		do_action( 'mainwp_pagefooter_post', 'Draft' );
	}

	/**
	 * Render Pages Drafts pages
	 *
	 * Renders the posts drafts page.
	 */
	public function render_draft_page() {
		do_action( 'mainwp_pageheader_page', 'Draft' );
		echo self::render_drafts_list( 'bulkpage' );
		do_action( 'mainwp_pagefooter_page', 'Draft' );
	}

	/**
	 * Render Drafts table
	 *
	 * Renders the Drafts table.
	 *
	 * @param string $post_type Post type.
	 */
	public static function render_drafts_list( $post_type ) {
		if ( 'bulkpage' != $post_type && 'bulkpost' != $post_type ) {
			return;
		}

		$add_new_link = 'PostBulkAdd';
		if ( 'bulkpage' == $post_type ) {
			$add_new_link = 'PageBulkAdd';
		}

		$args = array(
			'post_type'      => $post_type,
			'post_status'    => array( 'draft' ),
			'post_parent'    => null,
			'orderby'        => 'post_modified',
			'order'          => 'desc',
			'posts_per_page' => -1,
			'meta_key'       => '_saved_as_draft',
			'meta_value'     => 'yes',
		);

		$draft_posts = get_posts( $args );
		?>
		<div class="mainwp-actions-bar">
			<div class="ui grid">
				<div class="ui two column row">
					<div class="column ui mini form">
						<select class="ui dropdown" id="mainwp-post-plus-bulk-actions">
							<option value="none"><?php _e( 'Bulk Actions', 'mainwp-post-plus-extension' ); ?></option>
							<option value="publish-selected"><?php _e( 'Publish', 'mainwp-post-plus-extension' ); ?></option>
							<option value="preview-selected"><?php _e( 'Preview', 'mainwp-post-plus-extension' ); ?></option>
							<option value="delete-selected"><?php _e( 'Delete', 'mainwp-post-plus-extension' ); ?></option>
						</select>
						<input type="button" name="" id="mainwp-post-plus-bulk-actions-button" class="ui basic mini button" value="<?php _e( 'Apply', 'mainwp-post-plus-extension' ); ?>"/>
						<?php do_action( 'mainwp_post_plus_actions_bar_left' ); ?>
					</div>
					<div class="right aligned column">
						<a href="admin.php?page=<?php echo $add_new_link; ?>" class="ui mini green button"><?php _e( 'Create New', 'mainwp-post-plus-extension' ); ?></a>
						<?php do_action( 'mainwp_post_plus_actions_bar_right' ); ?>
					</div>
				</div>
			</div>
		</div>
		<div class="ui segment" id="mainwp-post-plus-drafts">
			<table class="ui single line table" id="mainwp-post-plus-drafts-table" style="width:100%">
				<thead>
					<tr>
						<th class="collapsing no-sort check-column"><span class="ui checkbox"><input type="checkbox" id="cb-select-all-top"></span></th>
						<th id="title"><?php _e( 'Title', 'mainwp-post-plus-extension' ); ?></th>
						<th id="author"><?php _e( 'Author', 'mainwp-post-plus-extension' ); ?></th>
						<th id="categories"><?php _e( 'Categories', 'mainwp-post-plus-extension' ); ?></th>
						<th id="tags"><?php _e( 'Tags', 'mainwp-post-plus-extension' ); ?></th>
						<th id="last-edit"><?php _e( 'Last Edit', 'mainwp-post-plus-extension' ); ?></th>
						<th id="actions" class="collapsing no-sort not-selectable"></th>
					</tr>
				</thead>
				<tbody>
					<?php if ( is_array( $draft_posts ) && count( $draft_posts ) > 0 ) : ?>
						<?php self::render_drafts_list_table_rows( $draft_posts, $post_type ); ?>
					<?php endif; ?>
				</tbody>
				<tfoot>
					<tr>
						<th class="collapsing no-sort check-column"><span class="ui checkbox"><input type="checkbox" id="cb-select-all-bottom"></span></th>
						<th><?php _e( 'Title', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Author', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Categories', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Tags', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Last Edit', 'mainwp-post-plus-extension' ); ?></th>
						<th class="collapsing no-sort"></th>
					</tr>
				</tfoot>
			</table>
			<?php self::render_screen_options(); ?>
			<script type="text/javascript">
			jQuery( document ).ready( function( $ ) {
				$post_plus_drafts_table = jQuery( '#mainwp-post-plus-drafts-table' ).DataTable( {
					"stateSave": true,
					"stateDuration": 0, // forever
					"scrollX": true,
					"colReorder" : { columns: ":not(.check-column):not(:last-child)" },
					"lengthMenu": [ [10, 25, 50, 100, -1], [10, 25, 50, 100, "All"] ],
					"columnDefs": [ { "targets": 'no-sort', "orderable": false } ],
					"order": [ [ 1, "asc" ] ],
					"language": { "emptyTable": "No saved drafts found." },
					"drawCallback": function( settings ) {
						jQuery('#mainwp-post-plus-drafts-table .ui.checkbox').checkbox();
						jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown').dropdown();
						mainwp_datatable_fix_menu_overflow();
					},
					'select': {
						items: 'row',
						style: 'multi+shift',
						selector: 'tr>td:not(.not-selectable)'
					},
				} ).on('select', function (e, dt, type, indexes) {
					if( 'row' == type ){
						dt.rows(indexes)
						.nodes()
						.to$().find('td.check-column .ui.checkbox' ).checkbox('set checked');
					}
				}).on('deselect', function (e, dt, type, indexes) {
					if( 'row' == type ){
						dt.rows(indexes)
						.nodes()
						.to$().find('td.check-column .ui.checkbox' ).checkbox('set unchecked');
					}
				}).on( 'columns-reordered', function ( e, settings, details ) {
					console.log('columns-reordered');
					setTimeout(() => {
						jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown' ).dropdown();
						jQuery( '#mainwp-post-plus-drafts-table .ui.checkbox' ).checkbox();
						mainwp_datatable_fix_menu_overflow('#mainwp-post-plus-drafts-table' );
					}, 1000);
				} );

				jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown' ).dropdown();
				jQuery( '#mainwp-post-plus-drafts-table .ui.checkbox' ).checkbox();
				mainwp_datatable_fix_menu_overflow('#mainwp-post-plus-drafts-table' );

				_init_post_plus_screen = function() {
					jQuery( '#mainwp-post-plus-screen-options-modal input[type=checkbox][id^="mainwp_show_column_"]' ).each( function() {
						var check_id = jQuery( this ).attr( 'id' );
						col_id = check_id.replace( "mainwp_show_column_", "" );
						try {
							$post_plus_drafts_table.column( '#' + col_id ).visible( jQuery( this ).is( ':checked' ) );
						} catch( err ) {
							// to fix js error.
	}
					} );
				};
				_init_post_plus_screen();

				mainwp_post_plus_screen_options = function () {
					jQuery( '#mainwp-post-plus-screen-options-modal' ).modal( {
						allowMultiple: true,
						onHide: function () {
						}
					} ).modal( 'show' );

					jQuery( '#post-plus-screen-options-form' ).submit( function() {
						if ( jQuery('input[name=reset_post_plus_columns_order]').attr( 'value' ) == 1 ) {
							jQuery( '#mainwp-post-plus-drafts-table' ).colReorder.reset();
		}
						jQuery( '#mainwp-post-plus-screen-options-modal' ).modal( 'hide' );
					} );
					return false;
				};
			} );

			</script>
					</div>
			<?php
	}

	/**
	 * Render All Drafts table
	 *
	 * Renders the Drafts table.
	 */
	public static function render_all_drafts_list() {
		$args = array(
			'post_type'      => array( 'bulkpage', 'bulkpost' ),
			'post_status'    => array( 'draft' ),
			'post_parent'    => null,
			'orderby'        => 'post_modified',
			'order'          => 'desc',
			'posts_per_page' => -1,
			'meta_key'       => '_saved_as_draft',
			'meta_value'     => 'yes',
		);

		$draft_posts = get_posts( $args );
		?>
		<div class="mainwp-actions-bar">
			<div class="ui grid">
				<div class="ui two column row">
					<div class="column ui mini form">
						<select class="ui dropdown" id="mainwp-post-plus-bulk-actions">
							<option value="none"><?php _e( 'Bulk Actions', 'mainwp-post-plus-extension' ); ?></option>
							<option value="publish-selected"><?php _e( 'Publish', 'mainwp-post-plus-extension' ); ?></option>
							<option value="preview-selected"><?php _e( 'Preview', 'mainwp-post-plus-extension' ); ?></option>
							<option value="delete-selected"><?php _e( 'Delete', 'mainwp-post-plus-extension' ); ?></option>
						</select>
						<input type="button" name="" id="mainwp-post-plus-bulk-actions-button" class="ui basic mini button" value="<?php _e( 'Apply', 'mainwp-post-plus-extension' ); ?>"/>
						<?php do_action( 'mainwp_post_plus_actions_bar_left' ); ?>
					</div>
					<div class="right aligned column">
						<a href="admin.php?page=PostBulkAdd" class="ui mini green button"><?php _e( 'Create New Post', 'mainwp-post-plus-extension' ); ?></a>
						<a href="admin.php?page=PageBulkAdd" class="ui mini green button"><?php _e( 'Create New Page', 'mainwp-post-plus-extension' ); ?></a>
						<?php do_action( 'mainwp_post_plus_actions_bar_right' ); ?>
					</div>
				</div>
			</div>
		</div>
		<div class="ui segment" id="mainwp-post-plus-drafts">
			<table class="ui single line table" id="mainwp-post-plus-drafts-table" style="width:100%">
				<thead>
					<tr>
						<th class="collapsing no-sort check-column"><span class="ui checkbox"><input type="checkbox" id="cb-select-all-top"></span></th>
						<th id="title"><?php _e( 'Title', 'mainwp-post-plus-extension' ); ?></th>
						<th id="type"><?php _e( 'Type', 'mainwp-post-plus-extension' ); ?></th>
						<th id="author"><?php _e( 'Author', 'mainwp-post-plus-extension' ); ?></th>
						<th id="categories"><?php _e( 'Categories', 'mainwp-post-plus-extension' ); ?></th>
						<th id="tags"><?php _e( 'Tags', 'mainwp-post-plus-extension' ); ?></th>
						<th id="last-edit"><?php _e( 'Last Edit', 'mainwp-post-plus-extension' ); ?></th>
						<th id="actions" class="collapsing no-sort"></th>
					</tr>
				</thead>
				<tbody>
					<?php if ( is_array( $draft_posts ) && count( $draft_posts ) > 0 ) : ?>
						<?php foreach ( $draft_posts as $post ) : ?>
							<?php $author_info = get_userdata( $post->post_author ); ?>
							<?php

							$post_type = get_post_type( $post );
							$type_text = ( $post_type == 'bulkpost' ) ? 'Post' : 'Page';
							?>
							<tr id="post-<?php echo intval( $post->ID ); ?>" post-id="<?php echo intval( $post->ID ); ?>">
								<td class="check-column"><span class="ui checkbox"><input type="checkbox" value="1" name="post[]"></span></td>
								<td><a href="?page=<?php echo ( $post_type == 'bulkpost' ? 'PostBulkEdit' : 'PageBulkEdit' ); ?>&post_id=<?php echo intval( $post->ID ); ?>"><?php echo esc_html( $post->post_title ); ?></a></td>
								<td><?php echo $type_text; ?></td>
								<td><?php echo $author_info->user_login; ?></td>
								<td><?php echo self::column_categories( $post, 'categories' ); ?></td>
								<td><?php echo self::column_categories( $post, 'tags' ); ?></td>
								<td><?php echo self::column_date( $post ); ?></td>
								<td class="not-selectable">
									<div class="ui right pointing dropdown icon mini basic green button">
										<a href="javascript:void(0)"><i class="ellipsis horizontal icon"></i></a>
										<div class="menu">
											<a href="?page=<?php echo ( $post_type == 'bulkpost' ? 'PostBulkEdit' : 'PageBulkEdit' ); ?>&post_id=<?php echo intval( $post->ID ); ?>" class="item"><?php _e( 'Edit' ); ?></a>
											<a href="#" class="item mainwp-post-plus-publish-action"><?php _e( 'Publish' ); ?></a>
											<a href="<?php echo get_permalink( $post->ID ) . '&preview=true'; ?>" class="item mainwp-post-plus-preview-action" target="_blank" ><?php _e( 'Preview' ); ?></a>
											<a href="#" class="item mainwp-post-plus-delete-action" ><?php _e( 'Delete', 'mainwp-post-plus-extension' ); ?></a>
										</div>
									</div>
								</td>
							</tr>
						<?php endforeach; ?>
					<?php endif; ?>
				</tbody>
				<tfoot>
					<tr>
						<th class="collapsing no-sort check-column"><span class="ui checkbox"><input type="checkbox" id="cb-select-all-bottom"></span></th>
						<th><?php _e( 'Title', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Type', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Author', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Categories', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Tags', 'mainwp-post-plus-extension' ); ?></th>
						<th><?php _e( 'Last Edit', 'mainwp-post-plus-extension' ); ?></th>
						<th class="collapsing no-sort"></th>
					</tr>
				</tfoot>
			</table>
			<?php self::render_screen_options(); ?>
			<script type="text/javascript">
				jQuery( document ).ready( function( $ ) {
					$post_plus_drafts_table = jQuery( '#mainwp-post-plus-drafts-table' ).DataTable( {
						"stateSave": true,
						"stateDuration": 0, // forever
						"scrollX": true,
						"colReorder" : { columns: ":not(.check-column):not(:last-child)" },
						"lengthMenu": [ [10, 25, 50, 100, -1], [10, 25, 50, 100, "All"] ],
						"columnDefs": [ { "targets": 'no-sort', "orderable": false } ],
						"order": [ [ 1, "asc" ] ],
						"language": { "emptyTable": "No saved drafts found." },
						"drawCallback": function( settings ) {
							jQuery('#mainwp-post-plus-drafts-table .ui.checkbox').checkbox();
							jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown').dropdown();
							mainwp_datatable_fix_menu_overflow();
						},
						'select': {
							items: 'row',
							style: 'multi+shift',
							selector: 'tr>td:not(.not-selectable)'
						},
					} ).on('select', function (e, dt, type, indexes) {
						if( 'row' == type ){
							dt.rows(indexes)
							.nodes()
							.to$().find('td.check-column .ui.checkbox' ).checkbox('set checked');
						}
					}).on('deselect', function (e, dt, type, indexes) {
						if( 'row' == type ){
							dt.rows(indexes)
							.nodes()
							.to$().find('td.check-column .ui.checkbox' ).checkbox('set unchecked');
						}
					}).on( 'columns-reordered', function ( e, settings, details ) {
						console.log('columns-reordered');
						setTimeout(() => {
							jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown' ).dropdown();
							jQuery( '#mainwp-post-plus-drafts-table .ui.checkbox' ).checkbox();
							mainwp_datatable_fix_menu_overflow('#mainwp-post-plus-drafts-table' );
						}, 1000);
					} );

					jQuery( '#mainwp-post-plus-drafts-table .ui.dropdown' ).dropdown();
					jQuery( '#mainwp-post-plus-drafts-table .ui.checkbox' ).checkbox();
					mainwp_datatable_fix_menu_overflow('#mainwp-post-plus-drafts-table' );

					_init_post_plus_screen = function() {
						jQuery( '#mainwp-post-plus-screen-options-modal input[type=checkbox][id^="mainwp_show_column_"]' ).each( function() {
							var check_id = jQuery( this ).attr( 'id' );
							col_id = check_id.replace( "mainwp_show_column_", "" );
							try {
								$post_plus_drafts_table.column( '#' + col_id ).visible( jQuery( this ).is( ':checked' ) );
							} catch( err ) {
								// to fix js error.
							}
						} );
					};
					_init_post_plus_screen();

					mainwp_post_plus_screen_options = function () {
						jQuery( '#mainwp-post-plus-screen-options-modal' ).modal( {
							allowMultiple: true,
							onHide: function () {
							}
						} ).modal( 'show' );

						jQuery( '#post-plus-screen-options-form' ).submit( function() {
							if ( jQuery('input[name=reset_post_plus_columns_order]').attr( 'value' ) == 1 ) {
								jQuery( '#mainwp-post-plus-drafts-table' ).colReorder.reset();
							}
							jQuery( '#mainwp-post-plus-screen-options-modal' ).modal( 'hide' );
						} );
						return false;
					};
				} );
			</script>
		</div>
		<?php
	}

	/**
	 * Render Drafts Table Rows
	 *
	 * Renders the Drafts table rows.
	 */
	public static function render_drafts_list_table_rows( $draft_posts, $post_type ) {

		$edit_link_slug = 'PostBulkEdit';
		if ( 'bulkpage' == $post_type ) {
			$edit_link_slug = 'PostBulkEdit';
		}

		foreach ( $draft_posts as $post ) {
			$author_info = get_userdata( $post->post_author );
			?>
			<tr id="post-<?php echo intval( $post->ID ); ?>" post-id="<?php echo intval( $post->ID ); ?>">
				<td class="check-column"><span class="ui checkbox"><input type="checkbox" value="1" name="post[]"></span></td>
				<td><a href="?page=<?php echo $edit_link_slug; ?>&post_id=<?php echo intval( $post->ID ); ?>"><?php echo esc_html( $post->post_title ); ?></a></td>
				<td><?php echo esc_html( $author_info->user_login ); ?></td>
				<td><?php echo self::column_categories( $post, 'categories' ); ?></td>
				<td><?php echo self::column_categories( $post, 'tags' ); ?></td>
				<td><?php echo self::column_date( $post ); ?></td>
				<td class="not-selectable">
					<div class="ui right pointing dropdown icon mini basic green button">
						<a href="javascript:void(0)"><i class="ellipsis horizontal icon"></i></a>
						<div class="menu">
							<a href="?page=<?php echo $edit_link_slug; ?>&post_id=<?php echo intval( $post->ID ); ?>" class="item"><?php _e( 'Edit' ); ?></a>
							<a href="#" class="item mainwp-post-plus-publish-action"><?php _e( 'Publish' ); ?></a>
							<a href="<?php echo get_permalink( $post->ID ) . '&preview=true'; ?>" class="item mainwp-post-plus-preview-action" target="_blank" ><?php _e( 'Preview' ); ?></a>
							<a href="#" class="item mainwp-post-plus-delete-action" ><?php _e( 'Delete' ); ?></a>
						</div>
					</div>
				</td>
			</tr>
			<?php
		}
	}

	/**
	 * Method screen_options()
	 *
	 * Create Screen Options button.
	 *
	 * @param mixed $input Screen options button HTML.
	 *
	 * @return mixed Screen sptions button.
	 */
	public function screen_options( $input ) {
		if ( isset( $_GET['page'] ) && ( 'Extensions-Mainwp-Post-Plus-Extension' == $_GET['page'] || 'PostDraft' == $_GET['page'] || 'PageDraft' == $_GET['page'] ) ) {
			if ( ! isset( $_GET['tab'] ) || 'dashboard' == $_GET['tab'] ) {
				$input .= '<a class="ui button basic icon" onclick="mainwp_post_plus_screen_options(); return false;" data-inverted="" data-position="bottom right" href="#" target="_blank" data-tooltip="' . esc_html__( 'Screen Options', 'mainwp-post-plus-extension' ) . '"><i class="cog icon"></i></a>';
			}
		}
		return $input;
	}

	/**
	 * Method handle_sites_screen_settings()
	 *
	 * Handle sites screen settings
	 */
	public function handle_sites_screen_settings() {
		if ( isset( $_POST['wp_nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wp_nonce'] ), 'PostPlusScrOptions' ) ) {
			$show_cols = array();
			foreach ( array_map( 'sanitize_text_field', wp_unslash( $_POST ) ) as $key => $val ) {
				if ( false !== strpos( $key, 'mainwp_show_column_' ) ) {
					$col               = str_replace( 'mainwp_show_column_', '', $key );
					$show_cols[ $col ] = 1;
				}
			}
			if ( isset( $_POST['show_columns_name'] ) ) {
				foreach ( array_map( 'sanitize_text_field', wp_unslash( $_POST['show_columns_name'] ) ) as $col ) {
					if ( ! isset( $show_cols[ $col ] ) ) {
						$show_cols[ $col ] = 0; // uncheck, hide columns.
					}
				}
			}
			$user = wp_get_current_user();
			if ( $user ) {
				update_user_option( $user->ID, 'mainwp_settings_show_post_plus_columns', $show_cols, true );
			}
		}
	}

	/**
	 * Render screen options.
	 *
	 * @return array Array of default column names.
	 */
	public static function render_screen_options() {

		$columns = self::get_columns();

		$show_cols = get_user_option( 'mainwp_settings_show_post_plus_columns' );

		if ( ! is_array( $show_cols ) ) {
			$show_cols = array();
		}

		?>
		<div class="ui modal" id="mainwp-post-plus-screen-options-modal">
		<i class="close icon"></i>
			<div class="header"><?php esc_html_e( 'Screen Options', 'mainwp-post-plus-extension' ); ?></div>
			<div class="scrolling content ui form">
				<form method="POST" action="" id="post-plus-screen-options-form" name="post_plus_screen_options_form">
					<?php wp_nonce_field( 'mainwp-admin-nonce' ); ?>
					<input type="hidden" name="wp_nonce" value="<?php echo wp_create_nonce( 'PostPlusScrOptions' ); ?>" />
						<div class="ui grid field">
							<label class="six wide column"><?php esc_html_e( 'Show columns', 'mainwp-post-plus-extension' ); ?></label>
							<div class="ten wide column">
								<ul class="mainwp_hide_wpmenu_checkboxes">
									<?php
									foreach ( $columns as $name => $title ) {
										?>
										<li>
											<div class="ui checkbox">
												<input type="checkbox"
												<?php
												$show_col = ! isset( $show_cols[ $name ] ) || ( 1 == $show_cols[ $name ] );
												if ( $show_col ) {
													echo 'checked="checked"';
												}
												?>
												id="mainwp_show_column_<?php echo esc_attr( $name ); ?>" name="mainwp_show_column_<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $name ); ?>">
												<label for="mainwp_show_column_<?php echo esc_attr( $name ); ?>" ><?php echo $title; ?></label>
												<input type="hidden" value="<?php echo esc_attr( $name ); ?>" name="show_columns_name[]" />
											</div>
										</li>
										<?php
									}
									?>
								</ul>
							</div>
					</div>
				</div>
				<div class="actions">
					<div class="ui two columns grid">
						<div class="left aligned column">
							<span data-tooltip="<?php esc_attr_e( 'Returns this page to the state it was in when installed. The feature also restores any column you have moved through the drag and drop feature on the page.', 'mainwp-post-plus-extension' ); ?>" data-inverted="" data-position="top center"><input type="button" class="ui button" name="reset" id="reset-post-plus-settings" value="<?php esc_attr_e( 'Reset Page', 'mainwp-post-plus-extension' ); ?>" /></span>
						</div>
						<div class="ui right aligned column">
							<input type="submit" class="ui green button" name="submit-post-plus-settings" id="submit-post-plus-settings" value="<?php esc_attr_e( 'Save Settings', 'mainwp-post-plus-extension' ); ?>" />
						</div>
					</div>
				</div>
				<input type="hidden" name="reset_post_plus_columns_order" value="0">
			</form>
		</div>
		<script type="text/javascript">
			jQuery( document ).ready( function () {
				jQuery( '#reset-post-plus-settings' ).on( 'click', function () {
					mainwp_confirm( __( 'Are you sure.' ), function() {
						jQuery( '.mainwp_hide_wpmenu_checkboxes input[id^="mainwp_show_column_"]' ).prop( 'checked', false );
						//default columns
						var cols = ['site','url','actions'];
						jQuery.each( cols, function ( index, value ) {
							jQuery( '.mainwp_hide_wpmenu_checkboxes input[id="mainwp_show_column_' + value + '"]' ).prop( 'checked', true );
						} );
						jQuery( 'input[name=reset_post_plus_columns_order]' ).attr( 'value', 1 );
						jQuery( '#submit-post-plus-settings' ).click();
					}, false, false, true );
					return false;
				} );
			} );
		</script>
		<?php
	}

	/**
	 * Get columns.
	 *
	 * @return array Array of column names.
	 */
	public static function get_columns() {
		return array(
			'title'      => __( 'Title', 'mainwp-post-plus-extension' ),
			'author'     => __( 'Author', 'mainwp-post-plus-extension' ),
			'categories' => __( 'Categories', 'mainwp-post-plus-extension' ),
			'tags'       => __( 'Tags', 'mainwp-post-plus-extension' ),
			'last-edit'  => __( 'Last Edit', 'mainwp-post-plus-extension' ),
			'actions'    => __( 'Action', 'mainwp-post-plus-extension' ),
		);
	}

	public static function format_time_stamp( $timestamp ) {
		return date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $timestamp );
	}

	public static function column_date( $post ) {
		if ( empty( $post ) ) {
			return '';
		}
		if ( '0000-00-00 00:00:00' == $post->post_date ) {
			$h_time = __( 'Unpublished', 'mainwp-post-plus-extension' );
		} else {
			$time   = mysql2date( 'U', $post->post_modified, false );
			$h_time = self::format_time_stamp( $time );
		}
		ob_start();
		echo $h_time;
		$output = ob_get_clean();
		return $output;
	}

	public static function column_categories( $post, $column_name ) {
		if ( empty( $post ) ) {
			return '';
		}
		$result = '';
		if ( 'categories' === $column_name ) {
			$saved_categories = get_post_meta( $post->ID, '_saved_draft_categories', true );
			$result           = rawurldecode( base64_decode( $saved_categories ) );
		} elseif ( 'tags' === $column_name ) {
			$saved_tags = get_post_meta( $post->ID, '_saved_draft_tags', true );
			$result     = base64_decode( $saved_tags );
		}
		return $result;
	}

	/**
	 * Method format_timestamp()
	 *
	 * Format the given timestamp.
	 *
	 * @param mixed $timestamp Timestamp to format.
	 *
	 * @return string Formatted timestamp.
	 */
	public static function format_timestamp( $timestamp ) {
		return date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $timestamp );
	}

	/**
	 * Method get_timestamp()
	 *
	 * Get time stamp in gmt_offset.
	 *
	 * @param mixed $timestamp Time stamp to convert.
	 *
	 * @return string Time stamp in general mountain time offset.
	 */
	public static function get_timestamp( $timestamp = false ) {
		if ( false === $timestamp ) {
			$timestamp = time();
		}
		$gmtOffset = get_option( 'gmt_offset' );

		return ( $gmtOffset ? ( $gmtOffset * HOUR_IN_SECONDS ) + $timestamp : $timestamp );
	}
}
